<?php
/**
 * Hatena Authentication API
 * see: http://auth.hatena.ne.jp/
 *
 * Yuichi Tateno <secondlife@no.spam@hatena.ne.jp>
 *
 * distributed under the MIT licence.
 *
 * example of use:
 * <code>
 * $hatena_auth = new Hatena_API_Auth('your api key', 'secret key');
 * $hatena_auth->uri_to_login();
 * // http://auth.hatena.ne.jp/auth?api_key=eab1f476f8f053ae4af086068017abe8&api_sig=9b192e80287aa75e4739105cfe7a9c45
 * $user = $hatena_auth->login($_GET['cert']);
 * </code>
 *
 *
 * modified by MATSUOKA Kohei <kmachu@gmail.com>
 *
 */
class Hatena_API_Auth
{
  var $VERSION = '0.1.0';
  var $api_key;
  var $secret;
  var $base_uri = 'http://auth.hatena.ne.jp';
  var $auth_path = '/auth';
  var $xml_path = '/api/auth.xml';

  function Hatena_API_Auth($api_key, $secret) {
    $this->api_key = $api_key;
    $this->secret = $secret;
  }

  function uri_to_login($param) {
    $uri = $this->base_uri . $this->auth_path;
    $query = array_merge($param, array('api_key' => $this->api_key));
    return $uri . $this->_get_query_string($query);
  }

  function login($cert) {
    $uri = $this->base_uri . $this->xml_path;
    $uri .= $this->_get_query_string( array(
      'api_key' => $this->api_key,
      'cert' => $cert,
    ));
    $handle = fopen($uri, 'rb');
    if(!$handle) {
        return false;
    }
    $contents = '';
    while (!feof($handle)) { $contents .= fread($handle, 8192); }
    fclose($handle);
    preg_match('@<name>([^<]+)</name>@', $contents, $matchs);
    if(!$matchs) {
      return false;
    }
    return $matchs[1];
  }

  function _get_query_string($request) {
    $query = array_merge(
      $request,
      array(
        'api_sig' => $this->api_sig($request)
      )
    );
    return $this->_querynize($query);
  }

  function api_sig($args) {
    $sig = $this->secret;
    $keys = array_keys($args);
    sort($keys);
    foreach($keys as $key) {
      $sig .= $key . $args[$key];
    }
    return md5($sig);
  }

  function _querynize($query) {
    $ary = array();
    foreach($query as $key => $value) {
      $ary[] = "${key}=${value}";
    }
    return '?' . join('&', $ary);
  }
}
?>
